/* This file is part of the GMOD package */

/* in cvt_period.c */
void period_to_note (int period, int *note, int *pitchbend);

/* in effects.c */
void set_speed (int parm);
void set_volslide (int channel, struct note_info *pat);
void set_slideto (int channel, struct note_info *pat);
void set_arpeg (int channel, int amount);
void set_vibrato (int channel, int amount);
void set_tremolo (int channel, int amount);

/* in init.c */
void init_voices (void);

/* int load_669.c */
int load_669_module (int mod_fd, char *name);

/* in load_mod.c */
int load_module (char *name);

/* in misc.c */
int panning (int ch);
void sync_time (void);
unsigned short intelize (unsigned short v);

/* in parse.c */
int parse_args (int argc, char *argv[]);

/* in play_mod.c */
void play_module (char *name);

/* in play_note.c */
int play_note (int channel, int position, int pattern, struct note_info *pat);

/* in play_voice.c */
void lets_play_voice (int channel, struct voice_info *v);
